@@ -241,23 +241,25 @@ public final class StringBuilder
      */
     @Override
     public StringBuilder append(char[] str) {
-        // DIFFBLUE MODEL LIBRARY: Handled internally by CBMC
+        // DIFFBLUE MODEL LIBRARY
+        String string = CProverString.ofCharArray(str, 0, str.length);
+        return append(string);
         // super.append(str);
         // return this;
-        return CProver.nondetWithNullForNotModelled();
     }
 
     /**
      * @throws IndexOutOfBoundsException {@inheritDoc}
-     *
-     * @diffblue.noSupport
+     * @diffblue.fullSupport
+     * @diffblue.untested
      */
     @Override
     public StringBuilder append(char[] str, int offset, int len) {
+        // DIFFBLUE MODEL LIBRARY
+        String string = new String(str, offset, len);
+        return append(string);
         // super.append(str, offset, len);
         // return this;
-        CProver.notModelled();
-        return CProver.nondetWithNullForNotModelled();
     }
 
     /**
@@ -485,7 +487,7 @@ public final class StringBuilder
         // return this;
         if ((offset < 0) || (offset > length()))
             throw new StringIndexOutOfBoundsException(offset);
-        return CProverString.insert(this, offset, b);
+        return CProverString.insert(this, offset, String.valueOf(b));
     }
 
     /**
@@ -500,7 +502,7 @@ public final class StringBuilder
         // return this;
         if ((offset < 0) || (offset > length()))
             throw new IndexOutOfBoundsException();
-        return CProverString.insert(this, offset, c);
+        return CProverString.insert(this, offset, String.valueOf(c));
     }
 
     /**
@@ -515,7 +517,7 @@ public final class StringBuilder
         // return this;
         if ((offset < 0) || (offset > length()))
             throw new StringIndexOutOfBoundsException(offset);
-        return CProverString.insert(this, offset, i);
+        return CProverString.insert(this, offset, String.valueOf(i));
     }
 
     /**
@@ -531,7 +533,7 @@ public final class StringBuilder
         // return this;
         if ((offset < 0) || (offset > length()))
             throw new StringIndexOutOfBoundsException(offset);
-        return CProverString.insert(this, offset, l);
+        return CProverString.insert(this, offset, String.valueOf(l));
     }
 
     /**
@@ -547,7 +549,7 @@ public final class StringBuilder
         // return this;
         if ((offset < 0) || (offset > length()))
             throw new StringIndexOutOfBoundsException(offset);
-        return CProverString.insert(this, offset, f);
+        return CProverString.insert(this, offset, String.valueOf(f));
     }
 
     /**
@@ -565,7 +567,7 @@ public final class StringBuilder
         // return this;
         if ((offset < 0) || (offset > length()))
             throw new StringIndexOutOfBoundsException(offset);
-        return CProverString.insert(this, offset, d);
+        return CProverString.insert(this, offset, String.valueOf(d));
     }
 
     /**
@@ -609,14 +611,22 @@ public final class StringBuilder
     }
 
     /**
-     * @diffblue.noSupport
+     * @diffblue.fullSupport
+     * @diffblue.untested
      */
     @Override
     public StringBuilder reverse() {
         // super.reverse();
         // return this;
-        CProver.notModelled();
-        return CProver.nondetWithNullForNotModelled();
+        int size = this.length();
+        if (size < 2)
+          return this;
+        String tmp = this.toString();
+        CProverString.delete(this, 0, size);
+        for (int i=size-1; i>=0; --i) {
+            this.append(CProverString.charAt(tmp, i));
+         }
+        return this;
     }
 
     /**
@@ -660,4 +670,38 @@ public final class StringBuilder
     //     value = (char[]) s.readObject();
     // }
 
+    // DIFFBLUE MODEL LIBRARY This should be inherited from AbstractStringBuilder
+    /**
+     * @diffblue.fullSupport
+     * @diffblue.untested
+     */
+    public void getChars(int srcBegin, int srcEnd, char dst[], int dstBegin) {
+        toString().getChars(srcBegin, srcEnd, dst, dstBegin);
+    }
+
+    /**
+     * @throws IndexOutOfBoundsException {@inheritDoc}
+     * @see    #length()
+     *
+     * @diffblue.fullSupport
+     */
+    @Override
+    public void setLength(int newLength) {
+        if (newLength < 0)
+            throw new StringIndexOutOfBoundsException(newLength);
+        CProverString.setLength(this, newLength);
+    }
+
+    /**
+     * @throws IndexOutOfBoundsException {@inheritDoc}
+     * @see    #length()
+     *
+     * @diffblue.fullSupport
+     */
+    @Override
+    public void setCharAt(int index, char ch) {
+        if ((index < 0) || (index >= this.length()))
+            throw new StringIndexOutOfBoundsException(index);
+        CProverString.setCharAt(this, index, ch);
+    }
 }
